local addon = LibStub("AceAddon-3.0"):GetAddon("BurlexUID");
local module = addon:NewModule("BottomBar");
----------------------------------------------------------------------------------------------------
local anchor, frame = BUID_AnchorFrame, BurlexUID;
local round = function(num) -- rounds a number to 2 decimal places
	return floor( (num*10^2)+0.5) / (10^2);
end;
local updateMinimumScale = function()
	local minScale = floor(((UIParent:GetWidth()/2560)*10^2)+1) / (10^2);
	if buidChar.scale < minScale then buidChar.scale = minScale; end
end;
local updateBurlexScale = function() -- scales BurlexUID based on setting or screen size
	buidChar = buidChar or {};
	if (not buidChar.scale) then -- make sure the variable exists, and auto-configured based on screen size
		local width, height = string.match(GetCVar("gxResolution"),"(%d+).-(%d+)");
		if (tonumber(width) / tonumber(height) > 4/3) then buidChar.scale = 0.92;
		else buidChar.scale = 1.00; end
	end
	updateMinimumScale();
	if (buidChar.scale ~= round(BurlexUID:GetScale())) then	
		frame:SetScale(buidChar.scale);
	end
end;
local updateBurlexOffset = function() -- handles BurlexUID offset based on setting or fubar / titan
	local fubar,offset = 0;
	if buidChar.offset then
		offset = max(buidChar.offset,1);
	else
		for i = 1,4 do
			if (_G["FuBarFrame"..i] and _G["FuBarFrame"..i]:IsVisible()) then
				local bar = _G["FuBarFrame"..i];
				local point = bar:GetPoint(1);
				if point == "BOTTOMLEFT" then fubar = fubar + bar:GetHeight(); 	end
			end
		end
		offset = max(fubar,1);
	end
	if (offset ~= round(anchor:GetHeight())) then anchor:SetHeight(offset); end
end;
local updateBurlexViewport = function() -- handles viewport offset based on settings
	WorldFrame:SetPoint("BOTTOMRIGHT",frame,"TOPRIGHT",0,-5);
end;
local updateBattlefieldMinimap = function()
	if ( BattlefieldMinimapTab and not BattlefieldMinimapTab:IsUserPlaced() ) then
		BattlefieldMinimapTab:ClearAllPoints()
		BattlefieldMinimapTab:SetPoint("RIGHT", "UIParent", "RIGHT",-144,150);
	end
end
----------------------------------------------------------------------------------------------------
function module:OnInitialize()
	do -- default interface modifications
		FramerateLabel:ClearAllPoints(); FramerateLabel:SetPoint("TOP", "WorldFrame", "TOP", -15, -50);
		MainMenuBar:Hide();
		hooksecurefunc(UIParent,"Hide",function() WorldFrame:SetPoint("BOTTOMRIGHT"); end);
		hooksecurefunc(UIParent,"Show",function()  updateBurlexViewport(); end);
		hooksecurefunc("updateContainerFrameAnchors",function() -- fix bag offsets
			local frame, xOffset, yOffset, screenHeight, freeScreenHeight, leftMostPoint, column
			local screenWidth = GetScreenWidth()
			local containerScale = 1
			local leftLimit = 0
			if ( BankFrame:IsShown() ) then
				leftLimit = BankFrame:GetRight() - 25
			end
			while ( containerScale > CONTAINER_SCALE ) do
				screenHeight = GetScreenHeight() / containerScale
				-- Adjust the start anchor for bags depending on the multibars
				xOffset = 1 / containerScale
				yOffset = 155;
				-- freeScreenHeight determines when to start a new column of bags
				freeScreenHeight = screenHeight - yOffset
				leftMostPoint = screenWidth - xOffset
				column = 1
				local frameHeight
				for index, frameName in ipairs(ContainerFrame1.bags) do
					frameHeight = getglobal(frameName):GetHeight()
					if ( freeScreenHeight < frameHeight ) then
						-- Start a new column
						column = column + 1
						leftMostPoint = screenWidth - ( column * CONTAINER_WIDTH * containerScale ) - xOffset
						freeScreenHeight = screenHeight - yOffset
					end
					freeScreenHeight = freeScreenHeight - frameHeight - VISIBLE_CONTAINER_SPACING
				end
				if ( leftMostPoint < leftLimit ) then
					containerScale = containerScale - 0.01
				else
					break
				end
			end
			if ( containerScale < CONTAINER_SCALE ) then
				containerScale = CONTAINER_SCALE
			end
			screenHeight = GetScreenHeight() / containerScale
			-- Adjust the start anchor for bags depending on the multibars
			xOffset = 1 / containerScale
			yOffset = 1
			-- freeScreenHeight determines when to start a new column of bags
			freeScreenHeight = screenHeight - yOffset
			column = 0
			for index, frameName in ipairs(ContainerFrame1.bags) do
				frame = getglobal(frameName)
				frame:SetScale(containerScale)
				if ( index == 1 ) then
					-- First bag
					frame:SetPoint("BOTTOMRIGHT", frame:GetParent(), "BOTTOMRIGHT", -xOffset, (yOffset + (buidChar.offset or 1)) * (buidChar.scale or 1) )
				elseif ( freeScreenHeight < frame:GetHeight() ) then
					-- Start a new column
					column = column + 1
					freeScreenHeight = screenHeight - yOffset
					frame:SetPoint("BOTTOMRIGHT", frame:GetParent(), "BOTTOMRIGHT", -(column * CONTAINER_WIDTH) - xOffset, yOffset )
				else
					-- Anchor to the previous bag
					frame:SetPoint("BOTTOMRIGHT", ContainerFrame1.bags[index - 1], "TOPRIGHT", 0, CONTAINER_SPACING)
				end
				freeScreenHeight = freeScreenHeight - frame:GetHeight() - VISIBLE_CONTAINER_SPACING
			end
		end);
		hooksecurefunc(GameTooltip,"SetPoint",function(tooltip,point,parent,rpoint) -- fix GameTooltip offset
			if (point == "BOTTOMRIGHT" and parent == "UIParent" and rpoint == "BOTTOMRIGHT") then
				tooltip:ClearAllPoints();
				tooltip:SetPoint("BOTTOMRIGHT","BurlexUID","TOPRIGHT",0,10);
			end
		end);
	end
	addon.options.args["maxres"] = {
		type = "execute", name = "Toggle Default Scales",
		desc = "toggles between widescreen and standard scales",
		func = function()
			if (InCombatLockdown()) then 
				addon:Print(ERR_NOT_IN_COMBAT);
			else
				if (buidChar.scale >= 0.78) or (buidChar.scale < 0.92) then
					buidChar.scale = 0.92;
				else
					buidChar.scale = 0.78;
				end
				addon:Print("Relative Scale set to "..buidChar.scale);
			end
		end
	};
	addon.options.args["scale"] = {
		type = "range", name = "Configure Scale",
		desc = "sets a specific scale for BurlexUID",
		set = function(info,val)
			if (InCombatLockdown()) then 
				addon:Print(ERR_NOT_IN_COMBAT);
			else
				buidChar.scale = min(1,round(val));
				updateMinimumScale();
				addon:Print("Relative Scale set to "..buidChar.scale);
			end
		end,
		get = function(info) return buidChar.scale; end
	};
	addon.options.args["offset"] = {
		type = "input",
		name = "Configure Offset",
		desc = "offsets the bottom bar automatically, or on a set value",
		set = function(info,val)
			if (InCombatLockdown()) then 
				addon:Print(ERR_NOT_IN_COMBAT);
			else
				if (val == "") or (val == "auto") then
					buidChar.offset = nil;
					addon:Print("Panel Offset set to AUTO");
				else
					val = tonumber(val);
					if (type(val) == "number") then
						val = max(0,val);						
						buidChar.offset = max(val+1,1);					
						addon:Print("Panel Offset set to "..val);
					end
				end
			end
		end,
		get = function(info) return buidChar.offset; end
	};
end
function module:OnEnable()
	anchor:SetFrameStrata("BACKGROUND"); anchor:SetFrameLevel(1);
	frame:SetFrameStrata("BACKGROUND"); frame:SetFrameLevel(1);
	
	hooksecurefunc("AchievementAlertFrame_ShowAlert",function() -- achivement alerts
		if (AchievementAlertFrame1) then AchievementAlertFrame1:SetPoint("BOTTOM",UIParent,"CENTER"); end
	end);
	hooksecurefunc("UIParent_ManageFramePositions",function()
		updateBattlefieldMinimap();
		if ( ArenaEnemyFrames ) then
			ArenaEnemyFrames:ClearAllPoints();
			ArenaEnemyFrames:SetPoint("RIGHT", UIParent, "RIGHT",0,40);
		end
		TutorialFrameAlertButton:SetParent(Minimap);
		TutorialFrameAlertButton:ClearAllPoints();
		TutorialFrameAlertButton:SetPoint("CENTER",Minimap,"TOP",-2,10);
		CastingBarFrame:ClearAllPoints();
		CastingBarFrame:SetPoint("BOTTOM",frame,"TOP",0,90);
	end);
	hooksecurefunc("ToggleBattlefieldMinimap",updateBattlefieldMinimap);
	
	updateBurlexScale();
	updateBurlexOffset();
	updateBurlexViewport();
	
	anchor:SetScript("OnUpdate",function()
		if (InCombatLockdown()) then return; end
		updateBurlexScale();
		updateBurlexOffset();
	end);
end
